/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_FLDR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_FLDR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_FLDR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_FLDR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_FLDR::Heavy_WSTD_FLDR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_bTMksBzv);
  numBytes += sEnv_init(&sEnv_gyqrCvSL, 512, 256);
  numBytes += sEnv_init(&sEnv_POl6N6ZS, 512, 256);
  numBytes += sEnv_init(&sEnv_MQwqCDDe, 512, 256);
  numBytes += sEnv_init(&sEnv_2gevfuSN, 512, 256);
  numBytes += sLine_init(&sLine_lbYcRX58);
  numBytes += sLine_init(&sLine_wEz0EHcD);
  numBytes += cVar_init_f(&cVar_i2f3XnZB, 0.0f);
  numBytes += cVar_init_f(&cVar_eBQhN8JX, 0.0f);
  numBytes += sVarf_init(&sVarf_Gv8Hmv9z, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_rqdeVqNx, 0.0f);
  numBytes += cIf_init(&cIf_Aot0GR7d, false);
  numBytes += cIf_init(&cIf_2TBw9dec, false);
  numBytes += cBinop_init(&cBinop_hZHV3V4I, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_ttTQ9hVb, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_Q87tNAXL, 0.0f);
  numBytes += cIf_init(&cIf_HSJlDoHN, false);
  numBytes += cIf_init(&cIf_Uldb1Ejk, false);
  numBytes += cBinop_init(&cBinop_bvRyV20Z, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_8fXzPuId, 1.0f);
  numBytes += sVarf_init(&sVarf_pBgBippe, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_56JtHdpe, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_7LYr1MJV, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_1K8CBKKG, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_FLDR::~Heavy_WSTD_FLDR() {
  sEnv_free(&sEnv_gyqrCvSL);
  sEnv_free(&sEnv_POl6N6ZS);
  sEnv_free(&sEnv_MQwqCDDe);
  sEnv_free(&sEnv_2gevfuSN);
}

HvTable *Heavy_WSTD_FLDR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_FLDR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xE42C892E: { // Fldr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_QPbDD9Zk_sendMessage);
      break;
    }
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_HRbAYLxn_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_RFkr5uo8_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_AobBGD3T_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_crcLVzMD_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_FLDR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Fldr";
        info->hash = 0xE42C892E;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 2: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_FLDR::cBinop_Bp7BcVSj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_06oinZnL_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_06oinZnL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_OPvZ17iE_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_OPvZ17iE_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_Ok96tob2_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_Ok96tob2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_4qBOCmqY_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_PfxuPVxf_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_PfxuPVxf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_nkNIlJD0_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_4qBOCmqY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_DN0lAl0S_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_DN0lAl0S_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_6CYQLiUq_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_nkNIlJD0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_JXT8LPSl_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_JXT8LPSl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_UuhTz2QF_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_6CYQLiUq_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_XHNDSvjh_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_UuhTz2QF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_DbIPpy6u_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_i2f3XnZB_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_K58lEI7Y_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_qArQSoo0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_HAq5FEGc_sendMessage(_c, 0, m);
  cMsg_opyXFlnD_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_eBQhN8JX_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_HGAEOVA6_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_rqdeVqNx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_hZHV3V4I, HV_BINOP_SUBTRACT, 0, m, &cBinop_hZHV3V4I_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_Aot0GR7d_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_Gv8Hmv9z, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_2TBw9dec_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_Gv8Hmv9z, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_gyqrCvSL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_rqdeVqNx, 0, m, &cVar_rqdeVqNx_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_POl6N6ZS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_puAcgrOW_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_Y1JzdKzd_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_nyaztyVS_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_hZHV3V4I_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_52lyEQlM_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_52lyEQlM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_6DrqKSki_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_6DrqKSki_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_cK43bAh5_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ZkjWUWIb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_qEBhCvXa_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_8WYnNbQt_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_qEBhCvXa_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Aot0GR7d, 1, m, &cIf_Aot0GR7d_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_8WYnNbQt_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_wjo4CrQr_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_YOMLp78c_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_2TBw9dec, 1, m, &cIf_2TBw9dec_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_wjo4CrQr_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_Aot0GR7d, 0, m, &cIf_Aot0GR7d_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_cK43bAh5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_2TBw9dec, 0, m, &cIf_2TBw9dec_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_nyaztyVS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_rqdeVqNx, 0, m, &cVar_rqdeVqNx_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_puAcgrOW_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_Y1JzdKzd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_hZHV3V4I, HV_BINOP_SUBTRACT, 1, m, &cBinop_hZHV3V4I_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_Q87tNAXL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_bvRyV20Z, HV_BINOP_SUBTRACT, 0, m, &cBinop_bvRyV20Z_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_HSJlDoHN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ttTQ9hVb, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_Uldb1Ejk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ttTQ9hVb, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_MQwqCDDe_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_Q87tNAXL, 0, m, &cVar_Q87tNAXL_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_2gevfuSN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_WU4ysHCf_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_XgK3n3tt_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_KksbzkqM_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_bvRyV20Z_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_gQ9mSFiY_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_gQ9mSFiY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_A3bu8gTU_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_A3bu8gTU_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_MmG1YQc2_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_AXqSEBLI_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_7eZFIxpZ_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_SQgEIy7Z_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_SQgEIy7Z_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_zUMvfEYR_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_7eZFIxpZ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_HSJlDoHN, 1, m, &cIf_HSJlDoHN_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_HztceMhx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Uldb1Ejk, 1, m, &cIf_Uldb1Ejk_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_zUMvfEYR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_HSJlDoHN, 0, m, &cIf_HSJlDoHN_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_MmG1YQc2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_Uldb1Ejk, 0, m, &cIf_Uldb1Ejk_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_XgK3n3tt_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_bvRyV20Z, HV_BINOP_SUBTRACT, 1, m, &cBinop_bvRyV20Z_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_KksbzkqM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_Q87tNAXL, 0, m, &cVar_Q87tNAXL_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_WU4ysHCf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cVar_8fXzPuId_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_OLNO6SwT_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_ZkjWUWIb_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_YOMLp78c_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_AXqSEBLI_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_HztceMhx_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_OLNO6SwT_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_8fXzPuId, 1, m, &cVar_8fXzPuId_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_XHNDSvjh_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_wEz0EHcD, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_DbIPpy6u_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_lbYcRX58, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cSend_K58lEI7Y_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_AobBGD3T_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_AJbIPQrD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_i2f3XnZB, 0, m, &cVar_i2f3XnZB_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_opyXFlnD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_AJbIPQrD_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_HGAEOVA6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_bTMksBzv, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_HAq5FEGc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_eBQhN8JX, 0, m, &cVar_eBQhN8JX_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_8fXzPuId, 0, m, &cVar_8fXzPuId_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_AobBGD3T_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_Bp7BcVSj_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_RFkr5uo8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_AJbIPQrD_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_crcLVzMD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_qArQSoo0_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_HRbAYLxn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_8fXzPuId, 0, m, &cVar_8fXzPuId_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_QPbDD9Zk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_eBQhN8JX, 0, m, &cVar_eBQhN8JX_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_FLDR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_pBgBippe, VIf(I0));
    __hv_line_f(&sLine_bTMksBzv, VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I0), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_floor_f(VIf(Bf2), VOf(Bf1));
    __hv_sub_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_abs_f(VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf2), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_gyqrCvSL, VIf(Bf3), &sEnv_gyqrCvSL_sendMessage);
    sEnv_process(this, &sEnv_POl6N6ZS, VIf(I0), &sEnv_POl6N6ZS_sendMessage);
    __hv_varread_f(&sVarf_Gv8Hmv9z, VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_56JtHdpe, VIf(Bf1));
    __hv_varwrite_f(&sVarf_7LYr1MJV, VIf(I1));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I1), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_floor_f(VIf(Bf0), VOf(Bf1));
    __hv_sub_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_abs_f(VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf0), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_MQwqCDDe, VIf(Bf3), &sEnv_MQwqCDDe_sendMessage);
    sEnv_process(this, &sEnv_2gevfuSN, VIf(I1), &sEnv_2gevfuSN_sendMessage);
    __hv_varread_f(&sVarf_ttTQ9hVb, VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_1K8CBKKG, VIf(Bf1));
    __hv_varread_f(&sVarf_7LYr1MJV, VOf(Bf1));
    __hv_line_f(&sLine_lbYcRX58, VOf(Bf3));
    __hv_varread_f(&sVarf_1K8CBKKG, VOf(Bf0));
    __hv_line_f(&sLine_wEz0EHcD, VOf(Bf2));
    __hv_mul_f(VIf(Bf0), VIf(Bf2), VOf(Bf0));
    __hv_fma_f(VIf(Bf1), VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_add_f(VIf(Bf0), VIf(O1), VOf(O1));
    __hv_varread_f(&sVarf_pBgBippe, VOf(Bf0));
    __hv_varread_f(&sVarf_56JtHdpe, VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf2), VOf(Bf2));
    __hv_add_f(VIf(Bf2), VIf(O0), VOf(O0));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_FLDR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_FLDR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
