/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvControlIf.h"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvControlBinop.h"
#include "HvSignalLine.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_Bp7BcVSj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_06oinZnL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OPvZ17iE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ok96tob2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_PfxuPVxf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_4qBOCmqY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DN0lAl0S_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_nkNIlJD0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JXT8LPSl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6CYQLiUq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_UuhTz2QF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_i2f3XnZB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qArQSoo0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_eBQhN8JX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_rqdeVqNx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Aot0GR7d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_2TBw9dec_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_gyqrCvSL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_POl6N6ZS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hZHV3V4I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_52lyEQlM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6DrqKSki_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZkjWUWIb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qEBhCvXa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8WYnNbQt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YOMLp78c_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_wjo4CrQr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cK43bAh5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_nyaztyVS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_puAcgrOW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Y1JzdKzd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Q87tNAXL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_HSJlDoHN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Uldb1Ejk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_MQwqCDDe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_2gevfuSN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bvRyV20Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gQ9mSFiY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_A3bu8gTU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AXqSEBLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SQgEIy7Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7eZFIxpZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HztceMhx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_zUMvfEYR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MmG1YQc2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XgK3n3tt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KksbzkqM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_WU4ysHCf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8fXzPuId_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OLNO6SwT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XHNDSvjh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_DbIPpy6u_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_K58lEI7Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AJbIPQrD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_opyXFlnD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HGAEOVA6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HAq5FEGc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_AobBGD3T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_RFkr5uo8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_crcLVzMD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_HRbAYLxn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_QPbDD9Zk_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_bTMksBzv;
  SignalEnvelope sEnv_gyqrCvSL;
  SignalEnvelope sEnv_POl6N6ZS;
  SignalEnvelope sEnv_MQwqCDDe;
  SignalEnvelope sEnv_2gevfuSN;
  SignalLine sLine_lbYcRX58;
  SignalLine sLine_wEz0EHcD;
  ControlBinop cBinop_Bp7BcVSj;
  ControlBinop cBinop_06oinZnL;
  ControlBinop cBinop_OPvZ17iE;
  ControlBinop cBinop_Ok96tob2;
  ControlBinop cBinop_DN0lAl0S;
  ControlBinop cBinop_JXT8LPSl;
  ControlBinop cBinop_6CYQLiUq;
  ControlBinop cBinop_UuhTz2QF;
  ControlVar cVar_i2f3XnZB;
  ControlVar cVar_eBQhN8JX;
  SignalVarf sVarf_Gv8Hmv9z;
  ControlVar cVar_rqdeVqNx;
  ControlIf cIf_Aot0GR7d;
  ControlIf cIf_2TBw9dec;
  ControlBinop cBinop_hZHV3V4I;
  ControlBinop cBinop_6DrqKSki;
  ControlBinop cBinop_ZkjWUWIb;
  ControlBinop cBinop_YOMLp78c;
  ControlBinop cBinop_cK43bAh5;
  SignalVarf sVarf_ttTQ9hVb;
  ControlVar cVar_Q87tNAXL;
  ControlIf cIf_HSJlDoHN;
  ControlIf cIf_Uldb1Ejk;
  ControlBinop cBinop_bvRyV20Z;
  ControlBinop cBinop_A3bu8gTU;
  ControlBinop cBinop_AXqSEBLI;
  ControlBinop cBinop_HztceMhx;
  ControlBinop cBinop_MmG1YQc2;
  ControlVar cVar_8fXzPuId;
  ControlBinop cBinop_OLNO6SwT;
  SignalVarf sVarf_pBgBippe;
  SignalVarf sVarf_56JtHdpe;
  SignalVarf sVarf_7LYr1MJV;
  SignalVarf sVarf_1K8CBKKG;
  ControlBinop cBinop_AJbIPQrD;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
